@@ -25,8 +25,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <dkp-client.h>
-#include <dkp-device.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
 #include "egg-debug.h"
 
@@ -43,6 +42,7 @@ static void     gpm_engine_finalize   (GObject	  *object);
 #define GPM_ENGINE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_ENGINE, GpmEnginePrivate))
 #define GPM_ENGINE_RESUME_DELAY		2*1000
 #define GPM_ENGINE_WARN_ACCURACY	20
+#define GPM_USE_COMPOSITE		0
 
 /* Left to convert:
  * 1. Recall data
@@ -53,6 +53,9 @@ struct GpmEnginePrivate
 	GConfClient		*conf;
 	DkpClient		*client;
 	GPtrArray		*array;
+#if GPM_USE_COMPOSITE
+	DkpDevice		*battery_composite;
+#endif
 	GpmPhone		*phone;
 	GpmIconPolicy		 icon_policy;
 	gchar			*previous_icon;
@@ -99,11 +102,16 @@ typedef enum {
  * gpm_engine_get_warning_csr:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_csr (GpmEngine *engine, const DkpObject *obj)
+gpm_engine_get_warning_csr (GpmEngine *engine, DkpDevice *device)
 {
-	if (obj->percentage < 26.0f)
+	gdouble percentage;
+
+	/* get device properties */
+	g_object_get (device, "percentage", &percentage, NULL);
+
+	if (percentage < 26.0f)
 		return GPM_ENGINE_WARNING_LOW;
-	else if (obj->percentage < 13.0f)
+	else if (percentage < 13.0f)
 		return GPM_ENGINE_WARNING_CRITICAL;
 	return GPM_ENGINE_WARNING_NONE;
 }
@@ -112,18 +120,23 @@ gpm_engine_get_warning_csr (GpmEngine *engine, const DkpObject *obj)
  * gpm_engine_get_warning_percentage:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_percentage (GpmEngine *engine, const DkpObject *obj)
+gpm_engine_get_warning_percentage (GpmEngine *engine, DkpDevice *device)
 {
+	gdouble percentage;
+
+	/* get device properties */
+	g_object_get (device, "percentage", &percentage, NULL);
+
 	/* this is probably an error condition */
-	if (obj->percentage == 0) {
+	if (percentage == 0) {
 		egg_warning ("percentage zero, something's gone wrong");
 		return GPM_ENGINE_WARNING_NONE;
 	}
-	if (obj->percentage <= engine->priv->action_percentage)
+	if (percentage <= engine->priv->action_percentage)
 		return GPM_ENGINE_WARNING_ACTION;
-	else if (obj->percentage <= engine->priv->critical_percentage)
+	else if (percentage <= engine->priv->critical_percentage)
 		return GPM_ENGINE_WARNING_CRITICAL;
-	else if (obj->percentage <= engine->priv->low_percentage)
+	else if (percentage <= engine->priv->low_percentage)
 		return GPM_ENGINE_WARNING_LOW;
 	return GPM_ENGINE_WARNING_NONE;
 }
@@ -132,19 +145,28 @@ gpm_engine_get_warning_percentage (GpmEngine *engine, const DkpObject *obj)
  * gpm_engine_get_warning_time:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_time (GpmEngine *engine, const DkpObject *obj)
+gpm_engine_get_warning_time (GpmEngine *engine, DkpDevice *device)
 {
+	DkpDeviceType type;
+	gint64 time_to_empty;
+
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "time-to-empty", &time_to_empty,
+		      NULL);
+
 	/* this is probably an error condition */
-	if (obj->time_to_empty == 0) {
-		egg_debug ("time zero, falling back to percentage for %s", dkp_device_type_to_text (obj->type));
-		return gpm_engine_get_warning_percentage (engine, obj);
+	if (time_to_empty == 0) {
+		egg_debug ("time zero, falling back to percentage for %s", dkp_device_type_to_text (type));
+		return gpm_engine_get_warning_percentage (engine, device);
 	}
 
-	if (obj->time_to_empty <= engine->priv->action_time)
+	if (time_to_empty <= engine->priv->action_time)
 		return GPM_ENGINE_WARNING_ACTION;
-	else if (obj->time_to_empty <= engine->priv->critical_time)
+	else if (time_to_empty <= engine->priv->critical_time)
 		return GPM_ENGINE_WARNING_CRITICAL;
-	else if (obj->time_to_empty <= engine->priv->low_time)
+	else if (time_to_empty <= engine->priv->low_time)
 		return GPM_ENGINE_WARNING_LOW;
 	return GPM_ENGINE_WARNING_NONE;
 }
@@ -158,41 +180,49 @@ gpm_engine_get_warning_time (GpmEngine *engine, const DkpObject *obj)
  * Return value: A GpmEngine state, e.g. GPM_ENGINE_WARNING_DISCHARGING
  **/
 static GpmEngineWarning
-gpm_engine_get_warning (GpmEngine *engine, const DkpObject *obj)
+gpm_engine_get_warning (GpmEngine *engine, DkpDevice *device)
 {
-	GpmEngineWarning type;
+	DkpDeviceType type;
+	DkpDeviceState state;
+	GpmEngineWarning warning_type;
+
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "state", &state,
+		      NULL);
 
 	/* default to no engine */
-	type = GPM_ENGINE_WARNING_NONE;
+	warning_type = GPM_ENGINE_WARNING_NONE;
 
-	if (obj->type == DKP_DEVICE_TYPE_MOUSE ||
-	    obj->type == DKP_DEVICE_TYPE_KEYBOARD) {
+	if (type == DKP_DEVICE_TYPE_MOUSE ||
+	    type == DKP_DEVICE_TYPE_KEYBOARD) {
 
-		type = gpm_engine_get_warning_csr (engine, obj);
+		warning_type = gpm_engine_get_warning_csr (engine, device);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_UPS ||
-		   obj->type == DKP_DEVICE_TYPE_PDA) {
+	} else if (type == DKP_DEVICE_TYPE_UPS ||
+		   type == DKP_DEVICE_TYPE_PDA) {
 
-		type = gpm_engine_get_warning_percentage (engine, obj);
+		warning_type = gpm_engine_get_warning_percentage (engine, device);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_PHONE) {
+	} else if (type == DKP_DEVICE_TYPE_PHONE) {
 
-		type = gpm_engine_get_warning_percentage (engine, obj);
+		warning_type = gpm_engine_get_warning_percentage (engine, device);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
+	} else if (type == DKP_DEVICE_TYPE_BATTERY) {
 		/* only use the time when it is accurate, and GConf is not disabled */
 		if (engine->priv->use_time_primary)
-			type = gpm_engine_get_warning_time (engine, obj);
+			warning_type = gpm_engine_get_warning_time (engine, device);
 		else
-			type = gpm_engine_get_warning_percentage (engine, obj);
+			warning_type = gpm_engine_get_warning_percentage (engine, device);
 	}
 
 	/* If we have no important engines, we should test for discharging */
-	if (type == GPM_ENGINE_WARNING_NONE) {
-		if (obj->state == DKP_DEVICE_STATE_DISCHARGING)
-			type = GPM_ENGINE_WARNING_DISCHARGING;
+	if (warning_type == GPM_ENGINE_WARNING_NONE) {
+		if (state == DKP_DEVICE_STATE_DISCHARGING)
+			warning_type = GPM_ENGINE_WARNING_DISCHARGING;
 	}
-	return type;
+	return warning_type;
 }
 
 /**
@@ -207,8 +237,7 @@ gpm_engine_get_summary (GpmEngine *engine)
 {
 	guint i;
 	GPtrArray *array;
-	const DkpDevice *device;
-	const DkpObject	*obj;
+	DkpDevice *device;
 	GString *tooltip = NULL;
 	gchar *part;
 
@@ -221,8 +250,7 @@ gpm_engine_get_summary (GpmEngine *engine)
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
-		obj = dkp_device_get_object (device);
-		part = gpm_devicekit_get_object_summary (obj);
+		part = gpm_devicekit_get_object_summary (device);
 		if (part != NULL)
 			g_string_append_printf (tooltip, "%s\n", part);
 		g_free (part);
@@ -242,32 +270,41 @@ gpm_engine_get_summary (GpmEngine *engine)
  * Returns the icon
  **/
 static gchar *
-gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType type, GpmEngineWarning warning, gboolean state)
+gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType device_type, GpmEngineWarning warning, gboolean use_state)
 {
 	guint i;
 	GPtrArray *array;
-	const DkpDevice *device;
-	const DkpObject	*obj;
+	DkpDevice *device;
 	GpmEngineWarning warning_temp;
+	DkpDeviceType type;
+	DkpDeviceState state;
+	gboolean is_present;
 
 	/* do we have specific device types? */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
-		obj = dkp_device_get_object (device);
+
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type,
+			      "state", &state,
+			      "is-present", &is_present,
+			      NULL);
+
 		warning_temp = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-warning-old"));
-		if (obj->type == type && obj->is_present) {
+		if (type == device_type && is_present) {
 			if (warning != GPM_ENGINE_WARNING_NONE) {
 				if (warning_temp == warning)
-					return gpm_devicekit_get_object_icon (obj);
+					return gpm_devicekit_get_object_icon (device);
 				continue;
 			}
-			if (state) {
-				if (obj->state == DKP_DEVICE_STATE_CHARGING || obj->state == DKP_DEVICE_STATE_DISCHARGING)
-					return gpm_devicekit_get_object_icon (obj);
+			if (use_state) {
+				if (state == DKP_DEVICE_STATE_CHARGING || state == DKP_DEVICE_STATE_DISCHARGING)
+					return gpm_devicekit_get_object_icon (device);
 				continue;
 			}
-			return gpm_devicekit_get_object_icon (obj);
+			return gpm_devicekit_get_object_icon (device);
 		}
 	}
 	return NULL;
@@ -471,24 +508,111 @@ gpm_engine_conf_key_changed_cb (GConfClient *conf, guint cnxn_id, GConfEntry *en
  * gpm_engine_device_check_capacity:
  **/
 static gboolean
-gpm_engine_device_check_capacity (GpmEngine *engine, const DkpDevice *device)
+gpm_engine_device_check_capacity (GpmEngine *engine, DkpDevice *device)
 {
-	const DkpObject	*obj;
 	gboolean ret;
+	DkpDeviceType type;
+	gdouble capacity;
 
-	obj = dkp_device_get_object (device);
-	if (obj->type != DKP_DEVICE_TYPE_BATTERY || obj->capacity > 50.0f)
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "capacity", &capacity,
+		      NULL);
+
+	if (type != DKP_DEVICE_TYPE_BATTERY || capacity > 50.0f)
 		return FALSE;
 
 	/* only emit this if specified in gconf */
 	ret = gconf_client_get_bool (engine->priv->conf, GPM_CONF_NOTIFY_LOW_CAPACITY, NULL);
 	if (ret) {
 		egg_debug ("** EMIT: low-capacity");
-		g_signal_emit (engine, signals [LOW_CAPACITY], 0, obj->type, (guint) obj->capacity);
+		g_signal_emit (engine, signals [LOW_CAPACITY], 0, device);
 	}
 	return TRUE;
 }
 
+#if GPM_USE_COMPOSITE
+/**
+ * gpm_engine_update_composite_device:
+ **/
+static DkpDevice *
+gpm_engine_update_composite_device (GpmEngine *engine)
+{
+	guint i;
+	gint64 time_to_empty = 0;
+	gint64 time_to_empty_total = 0;
+	gint64 time_to_full = 0;
+	gint64 time_to_full_total = 0;
+	gdouble percentage = 0;
+	gdouble percentage_total = 0;
+	guint battery_devices = 0;
+	gboolean is_charging = FALSE;
+	gboolean is_discharging = FALSE;
+	gboolean is_fully_charged = FALSE;
+	GPtrArray *array;
+	DkpDevice *device;
+	DkpDeviceState state;
+	DkpDeviceType type;
+
+	/* update the composite device */
+	array = engine->priv->array;
+	for (i=0;i<array->len;i++) {
+		device = g_ptr_array_index (engine->priv->array, i);
+		g_object_get (device,
+			      "type", &type,
+			      "time-to-empty", &time_to_empty,
+			      "time-to-full", &time_to_full,
+			      "percentage", &percentage,
+			      "state", &state,
+			      NULL);
+		if (type != DKP_DEVICE_TYPE_BATTERY)
+			continue;
+
+		egg_debug ("printing device %i", i);
+		dkp_device_print (device);
+
+		/* one of these will be charging or discharging */
+		if (state == DKP_DEVICE_STATE_CHARGING)
+			is_charging = TRUE;
+		if (state == DKP_DEVICE_STATE_DISCHARGING)
+			is_discharging = TRUE;
+		if (state == DKP_DEVICE_STATE_FULLY_CHARGED)
+			is_fully_charged = TRUE;
+		time_to_empty_total += time_to_empty;
+		time_to_full_total += time_to_full;
+		percentage_total += percentage;
+		battery_devices++;
+	}
+
+	/* use average percentage */
+	if (battery_devices > 0)
+		percentage_total /= battery_devices;
+
+	/* set composite state */
+	if (is_charging)
+		state = DKP_DEVICE_STATE_CHARGING;
+	else if (is_discharging)
+		state = DKP_DEVICE_STATE_DISCHARGING;
+	else if (is_fully_charged)
+		state = DKP_DEVICE_STATE_FULLY_CHARGED;
+	else
+		state = DKP_DEVICE_STATE_UNKNOWN;
+
+	egg_debug ("printing composite device");
+	g_object_set (engine->priv->battery_composite,
+		      "time-to-empty", time_to_empty_total,
+		      "time-to-full", time_to_full_total,
+		      "percentage", percentage_total,
+		      "state", state,
+		      NULL);
+	dkp_device_print (engine->priv->battery_composite);
+
+	/* return set composite device */
+	return engine->priv->battery_composite;
+}
+#endif
+
 /**
  * gpm_engine_device_add:
  **/
@@ -496,18 +620,39 @@ static void
 gpm_engine_device_add (GpmEngine *engine, DkpDevice *device)
 {
 	GpmEngineWarning warning;
-	const DkpObject *obj;
+	DkpDeviceState state;
+	DkpDeviceType type;
+#if GPM_USE_COMPOSITE
+	DkpDevice *composite;
+#endif
 
 	/* assign warning */
-	obj = dkp_device_get_object (device);
-	warning = gpm_engine_get_warning (engine, obj);
+	warning = gpm_engine_get_warning (engine, device);
 	g_object_set_data (G_OBJECT(device), "engine-warning-old", GUINT_TO_POINTER(warning));
 
 	/* check capacity */
 	gpm_engine_device_check_capacity (engine, device);
 
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "state", &state,
+		      NULL);
+
 	/* add old state for transitions */
-	g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(obj->state));
+	g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(state));
+
+#if GPM_USE_COMPOSITE
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		composite = gpm_engine_update_composite_device (engine);
+
+		/* get the same values for the composite device */
+		warning = gpm_engine_get_warning (engine, composite);
+		g_object_set_data (G_OBJECT(composite), "engine-warning-old", GUINT_TO_POINTER(warning));
+		g_object_get (composite, "state", &state, NULL);
+		g_object_set_data (G_OBJECT(composite), "engine-state-old", GUINT_TO_POINTER(state));
+	}
+#endif
 }
 
 /**
@@ -522,22 +667,27 @@ gpm_engine_coldplug_idle_cb (GpmEngine *engine)
 	gboolean has_ups = FALSE;
 	GpmPrefsServer *prefs_server;
 	DkpDevice *device;
-	const DkpObject	*obj;
+	DkpDeviceType type;
 
 	g_return_val_if_fail (engine != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), FALSE);
 
 	/* get array */
-	engine->priv->array = dkp_client_enumerate_devices (engine->priv->client);
+	engine->priv->array = dkp_client_enumerate_devices (engine->priv->client, NULL);
 
 	/* do we have specific device types? */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
-		obj = dkp_device_get_object (device);
-		if (obj->type == DKP_DEVICE_TYPE_BATTERY)
+
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type,
+			      NULL);
+
+		if (type == DKP_DEVICE_TYPE_BATTERY)
 			has_battery = TRUE;
-		else if (obj->type == DKP_DEVICE_TYPE_UPS)
+		else if (type == DKP_DEVICE_TYPE_UPS)
 			has_ups = TRUE;
 	}
 
@@ -596,56 +746,57 @@ gpm_engine_device_removed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 static void
 gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *engine)
 {
+	DkpDeviceType type;
 	DkpDeviceState state;
-	const DkpObject *obj;
+	DkpDeviceState state_old;
 	GpmEngineWarning warning_old;
 	GpmEngineWarning warning;
-	gboolean ret;
 
-	obj = dkp_device_get_object (device);
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "state", &state,
+		      NULL);
+
+	/* see if any interesting state changes have happened */
+	state_old = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-state-old"));
+	if (state_old != state) {
+		if (state == DKP_DEVICE_STATE_DISCHARGING) {
+			egg_debug ("** EMIT: discharging");
+			g_signal_emit (engine, signals [DISCHARGING], 0, device);
+		} else if (state == DKP_DEVICE_STATE_FULLY_CHARGED) {
+			egg_debug ("** EMIT: fully charged");
+			g_signal_emit (engine, signals [FULLY_CHARGED], 0, device);
+		}
+
+		/* save new state */
+		g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(state));
+	}
+
+#if GPM_USE_COMPOSITE
+	/* if battery then use composite device to cope with multiple batteries */
+	if (type == DKP_DEVICE_TYPE_BATTERY)
+		device = gpm_engine_update_composite_device (engine);
+#endif
 
 	/* check the warning state has not changed */
 	warning_old = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-warning-old"));
-	warning = gpm_engine_get_warning (engine, obj);
+	warning = gpm_engine_get_warning (engine, device);
 	if (warning != warning_old) {
 		if (warning == GPM_ENGINE_WARNING_LOW) {
 			egg_debug ("** EMIT: charge-low");
-			g_signal_emit (engine, signals [CHARGE_LOW], 0, obj);
+			g_signal_emit (engine, signals [CHARGE_LOW], 0, device);
 		} else if (warning == GPM_ENGINE_WARNING_CRITICAL) {
 			egg_debug ("** EMIT: charge-critical");
-			g_signal_emit (engine, signals [CHARGE_CRITICAL], 0, obj);
+			g_signal_emit (engine, signals [CHARGE_CRITICAL], 0, device);
 		} else if (warning == GPM_ENGINE_WARNING_ACTION) {
 			egg_debug ("** EMIT: charge-action");
-			g_signal_emit (engine, signals [CHARGE_ACTION], 0, obj);
+			g_signal_emit (engine, signals [CHARGE_ACTION], 0, device);
 		}
 		/* save new state */
 		g_object_set_data (G_OBJECT(device), "engine-warning-old", GUINT_TO_POINTER(warning));
 	}
 
-	/* see if any interesting state changes have happened */
-	state = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-state-old"));
-	if (state != obj->state) {
-
-		if (obj->state == DKP_DEVICE_STATE_DISCHARGING) {
-			/* only emit this if specified in gconf */
-			ret = gconf_client_get_bool (engine->priv->conf, GPM_CONF_NOTIFY_DISCHARGING, NULL);
-			if (ret) {
-				egg_debug ("** EMIT: discharging");
-				g_signal_emit (engine, signals [DISCHARGING], 0, obj->type);
-			}
-		} else if (obj->state == DKP_DEVICE_STATE_FULLY_CHARGED) {
-			/* only emit this if specified in gconf */
-			ret = gconf_client_get_bool (engine->priv->conf, GPM_CONF_NOTIFY_FULLY_CHARGED, NULL);
-			if (ret) {
-				egg_debug ("** EMIT: discharging");
-				g_signal_emit (engine, signals [FULLY_CHARGED], 0, obj->type);
-			}
-		}
-
-		/* save new state */
-		g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(obj->state));
-	}
-
 	gpm_engine_recalculate_state (engine);
 }
 
@@ -673,16 +824,18 @@ gpm_engine_get_devices (GpmEngine *engine)
 static void
 phone_device_added_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
-	DkpObject *obj;
 	DkpDevice *device;
 	device = dkp_device_new ();
 
 	egg_debug ("phone added %i", idx);
-	
-	obj = (DkpObject *) dkp_device_get_object (device);
-	obj->native_path = g_strdup_printf ("phone_%i", idx);
-	obj->is_rechargeable = TRUE;
-	obj->type = DKP_DEVICE_TYPE_PHONE;
+
+	/* get device properties */
+	g_object_set (device,
+		      "type", DKP_DEVICE_TYPE_PHONE,
+		      "is-rechargeable", TRUE,
+		      "native-path", g_strdup_printf ("dummy:phone_%i", idx),
+		      "is-present", TRUE,
+		      NULL);
 
 	/* state changed */
 	gpm_engine_device_add (engine, device);
@@ -698,14 +851,19 @@ phone_device_removed_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
 	guint i;
 	DkpDevice *device;
-	const DkpObject *obj;
+	DkpDeviceType type;
 
 	egg_debug ("phone removed %i", idx);
 
 	for (i=0; i<engine->priv->array->len; i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
-		obj = dkp_device_get_object (device);
-		if (obj->type == DKP_DEVICE_TYPE_PHONE) {
+
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type,
+			      NULL);
+
+		if (type == DKP_DEVICE_TYPE_PHONE) {
 			g_ptr_array_remove_index (engine->priv->array, i);
 			g_object_unref (device);
 			break;
@@ -724,17 +882,28 @@ phone_device_refresh_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
 	guint i;
 	DkpDevice *device;
-	DkpObject *obj;
+	DkpDeviceType type;
+	DkpDeviceState state;
+	gboolean is_present;
+	gdouble percentage;
 
 	egg_debug ("phone refresh %i", idx);
 
 	for (i=0; i<engine->priv->array->len; i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
-		obj = (DkpObject *) dkp_device_get_object (device);
-		if (obj->type == DKP_DEVICE_TYPE_PHONE) {
-			obj->is_present = gpm_phone_get_present (phone, idx);
-			obj->state = gpm_phone_get_on_ac (phone, idx) ? DKP_DEVICE_STATE_CHARGING : DKP_DEVICE_STATE_DISCHARGING;
-			obj->percentage = gpm_phone_get_percentage (phone, idx);
+
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type,
+			      "state", &state,
+			      "percentage", &percentage,
+			      "is-present", &is_present,
+			      NULL);
+
+		if (type == DKP_DEVICE_TYPE_PHONE) {
+			is_present = gpm_phone_get_present (phone, idx);
+			state = gpm_phone_get_on_ac (phone, idx) ? DKP_DEVICE_STATE_CHARGING : DKP_DEVICE_STATE_DISCHARGING;
+			percentage = gpm_phone_get_percentage (phone, idx);
 			break;
 		}
 	}
@@ -776,6 +945,17 @@ gpm_engine_init (GpmEngine *engine)
 	g_signal_connect (engine->priv->phone, "device-refresh",
 			  G_CALLBACK (phone_device_refresh_cb), engine);
 
+#if GPM_USE_COMPOSITE
+	/* create a fake virtual composite battery */
+	engine->priv->battery_composite = dkp_device_new ();
+	g_object_set (engine->priv->battery_composite,
+		      "type", DKP_DEVICE_TYPE_BATTERY,
+		      "is-rechargeable", TRUE,
+		      "native-path", "dummy:composite_battery",
+		      "is-present", TRUE,
+		      NULL);
+#endif
+
 	engine->priv->previous_icon = NULL;
 	engine->priv->previous_summary = NULL;
 
@@ -834,30 +1014,30 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, low_capacity),
-			      NULL, NULL, gpm_marshal_VOID__UINT_UINT,
-			      G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
+			      NULL, NULL, g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [PERHAPS_RECALL] =
 		g_signal_new ("perhaps-recall",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, perhaps_recall),
-			      NULL, NULL, gpm_marshal_VOID__UINT_STRING_STRING,
+			      NULL, NULL, gpm_marshal_VOID__POINTER_STRING_STRING,
 			      G_TYPE_NONE,
-			      3, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
+			      3, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING);
 	signals [FULLY_CHARGED] =
 		g_signal_new ("fully-charged",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, fully_charged),
 			      NULL, NULL, g_cclosure_marshal_VOID__UINT,
-			      G_TYPE_NONE, 1, G_TYPE_UINT);
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [DISCHARGING] =
 		g_signal_new ("discharging",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, discharging),
-			      NULL, NULL, g_cclosure_marshal_VOID__UINT,
-			      G_TYPE_NONE, 1, G_TYPE_UINT);
+			      NULL, NULL, g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [CHARGE_ACTION] =
 		g_signal_new ("charge-action",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -901,6 +1081,9 @@ gpm_engine_finalize (GObject *object)
 
 	g_object_unref (engine->priv->client);
 	g_object_unref (engine->priv->phone);
+#if GPM_USE_COMPOSITE
+	g_object_unref (engine->priv->battery_composite);
+#endif
 
 	g_free (engine->priv->previous_icon);
 	g_free (engine->priv->previous_summary);
