@@ -39,7 +39,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
-#include <dkp-device.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
 #include "egg-debug.h"
 
@@ -54,6 +54,7 @@
 static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 #define GPM_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_TRAY_ICON, GpmTrayIconPrivate))
+#define GPM_TRAY_ICON_CONNECT_TIMEOUT		10 /* s */
 
 struct GpmTrayIconPrivate
 {
@@ -177,7 +178,6 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	gchar *description = NULL;
 	GtkWidget *dialog;
 	GtkWidget *image;
-	const DkpObject	*obj;
 	const gchar *object_path;
 	DkpDevice *device = NULL;
 	gboolean ret;
@@ -196,12 +196,11 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 	}
 
 	device = dkp_device_new ();
-	ret = dkp_device_set_object_path (device, object_path);
+	ret = dkp_device_set_object_path (device, object_path, NULL);
 	if (!ret)
 		goto out;
-	obj = dkp_device_get_object (device);
-	icon_name = gpm_devicekit_get_object_icon (obj);
-	description = gpm_devicekit_get_object_description (obj);
+	icon_name = gpm_devicekit_get_object_icon (device);
+	description = gpm_devicekit_get_object_description (device);
 
 	image = gtk_image_new ();
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -255,12 +254,10 @@ gpm_tray_icon_suspend_cb (GtkMenuItem *item, gpointer data)
 static void
 gpm_tray_icon_show_statistics_cb (GtkMenuItem *item, gpointer data)
 {
-	gchar *path;
+	const gchar *command = "gnome-power-statistics";
 
-	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
-	if (g_spawn_command_line_async (path, NULL) == FALSE)
-		egg_warning ("Couldn't execute command: %s", path);
-	g_free (path);
+	if (g_spawn_command_line_async (command, NULL) == FALSE)
+		egg_warning ("Couldn't execute command: %s", command);
 }
 
 /**
@@ -402,8 +399,6 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 t
 	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
 	GtkWidget *item;
 	GtkWidget *image;
-	gchar *path;
-	gboolean ret;
 
 	egg_debug ("icon right clicked");
 
@@ -418,22 +413,15 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 t
 			  G_CALLBACK (gpm_tray_icon_show_preferences_cb), icon);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* this program is optional, and may be in an uninstalled subpackage */
-	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
-	ret = g_file_test (path, G_FILE_TEST_EXISTS);
-	g_free (path);
-
 	/* statistics */
-	if (ret) {
-		item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
-		image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
+	item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
+	image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* separator for HIG */
+	/* separator for HIG? */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -478,15 +466,22 @@ gpm_tray_icon_add_device (GpmTrayIcon *icon, GtkMenu *menu, const GPtrArray *arr
 	GtkWidget *item;
 	GtkWidget *image;
 	const gchar *object_path;
-	const DkpObject	*obj;
 	const gchar *desc;
-	const DkpDevice *device;
+	DkpDevice *device;
+	DkpDeviceType type_tmp;
+	gdouble percentage;
 
 	/* find type */
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (array, i);
-		obj = dkp_device_get_object (device);
-		if (obj->type != type)
+
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type_tmp,
+			      "percentage", &percentage,
+			      NULL);
+
+		if (type != type_tmp)
 			continue;
 
 		object_path = dkp_device_get_object_path (device);
@@ -495,11 +490,11 @@ gpm_tray_icon_add_device (GpmTrayIcon *icon, GtkMenu *menu, const GPtrArray *arr
 
 		/* generate the label */
 		desc = gpm_device_type_to_localised_text (type, 1);
-		label = g_strdup_printf ("%s (%.1f%%)", desc, obj->percentage);
+		label = g_strdup_printf ("%s (%.1f%%)", desc, percentage);
 		item = gtk_image_menu_item_new_with_label (label);
 
 		/* generate the image */
-		icon_name = gpm_devicekit_get_object_icon (obj);
+		icon_name = gpm_devicekit_get_object_icon (device);
 		image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 
@@ -618,6 +613,18 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 	}
 }
 
+/**
+ * gpm_tray_status_icon_handler_cb:
+ *
+ * This attaches any future notifications to the status icon if it exists
+ **/
+static gboolean
+gpm_tray_status_icon_handler_cb (GpmTrayIcon *icon)
+{
+	gpm_notify_use_status_icon (icon->priv->notify, icon->priv->status_icon);
+	return FALSE;
+}
+
 /**
  * gpm_tray_icon_init:
  *
@@ -642,7 +649,7 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 	icon->priv->conf = gconf_client_get_default ();
 	/* watch gnome-power-manager keys */
 	gconf_client_add_dir (icon->priv->conf, GPM_CONF_DIR,
-			      GCONF_CLIENT_PRELOAD_NONE, NULL);
+			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
 	gconf_client_notify_add (icon->priv->conf, GPM_CONF_DIR,
 				 (GConfClientNotifyFunc) gpm_conf_gconf_key_changed_cb,
 				 icon, NULL, NULL);
@@ -656,7 +663,9 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 				 "activate",
 				 G_CALLBACK (gpm_tray_icon_activate_cb),
 				 icon, 0);
-	gpm_notify_use_status_icon (icon->priv->notify, icon->priv->status_icon);
+
+	/* attach the status icon after a few seconds to allow for the session to load properly */
+	g_timeout_add_seconds (GPM_TRAY_ICON_CONNECT_TIMEOUT, (GSourceFunc) gpm_tray_status_icon_handler_cb, icon);
 
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
